/**
 * 
 */
package de.mpc.prolic.misc;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.ResourceBundle;

/**
 * @author Michael Kohl
 * This class is a validator for a given ProLiC configuration file. In particular,
 * usage of permitted values for certain keys in the configuration file is checked.
 * Detected inconsistencies will result in a shutdown of ProLiC.
 */
public class CheckConfiguration {
	/*******************************************************************************
	 * Constants
	 ******************************************************************************/
	private final String spacer = " ";
	//List of keys where default values are available within Prolic.java
	//section 'genericSettings'
	private final String def_colDelimIn="column_delimiter_input";
	private final String def_colDelimOut="column_delimiter_output";
	private final String def_fileType="file_type";
	private final String def_encoding_output="encoding_output";
	//section 'Seq. based comparison'
	private final String def_substitution_matrix_id="BLOSUM62";
	/*******************************************************************************
	 * Members
	 ******************************************************************************/
	private String nameConfFile; 
	//HashMaps with allowed values for specific sections
	private HashMap<String, HashMap<String, PermittedValues>> perValComplete;	//Stores all the permitted values of the configuration files
	private HashMap<String, PermittedValues> genericSettings; 					//Permitted values for each key of the section 'genericSettings' in the configuration file.
	private HashMap<String, PermittedValues> commandLineSettings; 				//Permitted values for each key of the section 'commandLineSettings' in the configuration file.
	private HashMap<String, PermittedValues> grouping; 							//Permitted values for each key of the section 'grouping' in the configuration file.
	private HashMap<String, PermittedValues> sequenceBasedComparison;			//Permitted values for each key of the section 'Seq. based comparison' in the configuration file.
	private HashMap<String, PermittedValues> peptideBasedComparison; 			//Permitted values for each key of the section 'Peptide based comparison' in the configuration file.
	ResourceBundle bundleMess = ResourceBundle.getBundle("de.mpc.prolic.resources.MessageBundle");		//A localized bundle for user information and for logging
	private HashMap<String, String> wrongValues;								//Map with wrong values for a given key
	private HashMap<String, String> AppliedDefaultValues;						//Map with wrong values for keys where default values are applied
	/**
	 * Empty constructor
	 */
	public CheckConfiguration(String nameConfFile_) {
		nameConfFile=nameConfFile_;
		perValComplete=new HashMap<String, HashMap<String, PermittedValues>>();
		genericSettings=new HashMap<String, PermittedValues>();
		commandLineSettings=new HashMap<String, PermittedValues>();
		grouping=new HashMap<String, PermittedValues>();
		sequenceBasedComparison=new HashMap<String, PermittedValues>();
		peptideBasedComparison=new HashMap<String, PermittedValues>();
		wrongValues=new HashMap<String, String>();
		AppliedDefaultValues=new HashMap<String, String>();
		init();
	}
	
	/**
	 * Preparation of the configuration check, i.e. generation of reference values. 
	 */
	private void init(){
		generateGenericReferences();
		generateComLineReferences();
		generateGroupReferences();
		generateSeqBasedReferences();
		generatePeptBasedReferences();	
	}
	
	/**
	 * The method validates all the values given in the ProLiC configuration file. If there is a single wrong setting within the file
	 * the file is considered incorrect and ProLiC will shut down. 
	 * @return a boolean that indicates correctness of settings considering a given ProLiC configuration file in total.
	 */
	public boolean checkCompleteConfigurationFile(){
		boolean success=true;
		boolean sectionSucess=true;
		sectionSucess=this.checkSectionGeneric("genericSettings");
		if(!sectionSucess){
			success=false;
		}
		sectionSucess=checkSection("commandLineSettings");
		if(!sectionSucess){
			success=false;
		}
		sectionSucess=checkSection("grouping");
		if(!sectionSucess){
			success=false;
		}
		sectionSucess=checkSectionSeqBased("Seq. based comparison");
		if(!sectionSucess){
			success=false;
		}
		sectionSucess=checkSection("Peptide based comparison");
		if(!sectionSucess){
			success=false;
		}
		return success;
	}
	
	/**
	 * Validates the values of the configuration file for a given section
	 * @param section
	 * @return a boolean that indicates correctness of settings in a specific section of the ProLiC configuration file.
	 */
	private boolean checkSection(String section){
		boolean success=true;
		Map<String, PermittedValues> aSectionMap;
		PermittedValues aPerVal;
		ArrayList <String> aPerValAsList;
		Iterator<String> it;
		String currKey;
		String givenValue;
		String mess = bundleMess.getString("m_access_error_settings")+" "+section+"."+spacer+bundleMess.getString("m_program_shutdown");
		Map <String, String> settings;
		boolean doShutdown=true;		
		settings=IO.readSectionFromIni(nameConfFile, section, true, mess, doShutdown); //the third boolean parameter has no consequences. ProLiC shutdown in case of IO exceptions.
		it=settings.keySet().iterator();
		while(it.hasNext()){
			currKey=it.next();
			givenValue=settings.get(currKey);
			aSectionMap=perValComplete.get(section);
			aPerVal=aSectionMap.get(currKey);			
			if(aPerVal!=null){//no checks required for these keys
				aPerValAsList=aPerVal.getPermittedValues();
				if(!aPerValAsList.contains(givenValue)){
					success=false;
					this.wrongValues.put(currKey, givenValue);
				}
			}
		}
		return success;
	}
	
	/**
	 * Checking the 'special' section 'genericSettings' of the configuration file.
	 * In this case some default values are available and ProLiC will not shut down if wrong
	 * values are specified. Instead, information about the used default values is given in the log file.
	 * @return a boolean that indicates correctness of settings in the 'genericSettings' section of the ProLiC configuration file.
	 */
	private boolean checkSectionGeneric(String sectionGeneric){
		boolean success=true;
		Iterator<String> it;
		String currKey;
		String givenValue;
		ArrayList<String> perVal;
		String mess = bundleMess.getString("m_access_error_settings")+" "+sectionGeneric+"."+spacer+bundleMess.getString("m_program_shutdown");
		Map <String, String> settings;
		boolean doShutdown=true;		
		settings=IO.readSectionFromIni(nameConfFile, sectionGeneric, true, mess, doShutdown); //the third boolean parameter has no consequences. ProLiC shutdown in case of IO exceptions.
		it=settings.keySet().iterator();
		while(it.hasNext()){
			currKey=it.next();
			if(perValComplete.get(sectionGeneric).get(currKey)!=null){//only if there are existing permitted values available for a given key a check is performed			
				givenValue=settings.get(currKey);
				perVal=perValComplete.get(sectionGeneric).get(currKey).getPermittedValues();
				if((currKey.equals(def_colDelimIn))||(currKey.equals(def_colDelimOut))||
						(currKey.equals(def_fileType))||(currKey.equals(def_encoding_output))){
					if(!perVal.contains(givenValue)){
						AppliedDefaultValues.put(currKey, givenValue);
					}				
				}else{
					if(!perVal.contains(givenValue)){
						success=false;
						wrongValues.put(currKey, givenValue);
					}				
				}
			}
		}
		return success;		
	}
	
	/**
	 * Checking the 'special' section 'Seq. based comparison' of the configuration file.
	 * In this case some default values are available and ProLiC will not shut down if wrong
	 * values are specified. Instead, information about the used default values is given in the log file.
	 * @return a boolean that indicates correctness of settings in the 'genericSettings' section of the ProLiC configuration file.
	 */
	private boolean checkSectionSeqBased(String sectionSeqBased){
		boolean success=true;
		Iterator<String> it;
		String currKey;
		String givenValue;
		ArrayList<String> perVal;
		String mess = bundleMess.getString("m_access_error_settings")+" "+sectionSeqBased+"."+spacer+bundleMess.getString("m_program_shutdown");
		Map <String, String> settings;
		boolean doShutdown=true;		
		settings=IO.readSectionFromIni(nameConfFile, sectionSeqBased, true, mess, doShutdown); //the third boolean parameter has no consequences. ProLiC shutdown in case of IO exceptions.
		it=settings.keySet().iterator();
		while(it.hasNext()){
			currKey=it.next();
			if(perValComplete.get(sectionSeqBased).get(currKey)!=null){//only if there are existing permitted values available for a given key a check is performed	
				givenValue=settings.get(currKey);
				perVal=perValComplete.get(sectionSeqBased).get(currKey).getPermittedValues();
				if((currKey.equals(this.def_substitution_matrix_id))){
					if(!perVal.contains(givenValue)){
						AppliedDefaultValues.put(currKey, givenValue);
					}				
				}else{
					if(!perVal.contains(givenValue)){
						success=false;
						wrongValues.put(currKey, givenValue);
					}				
				}
			}
		}
		return success;		
	}
	
	/**
	 * This method generates value references for the section 'genericSettings' of the configuration file.
	 */
	private void generateGenericReferences(){
		PermittedValues aPV;
		//column_delimiter_input
		aPV=new PermittedValues();
		aPV.addValue(",");
		aPV.addValue(";");
		aPV.addValue("\t");
		genericSettings.put("column_delimiter_input", aPV);
		//column_delimiter_output
		aPV=new PermittedValues();
		aPV.addValue(",");
		aPV.addValue(";");
		aPV.addValue("\t");
		genericSettings.put("column_delimiter_output", aPV);
		//file_type
		aPV=new PermittedValues();
		aPV.addValue("protein_information");
		genericSettings.put("file_type", aPV);
		//considerProteinNames
		aPV=new PermittedValues();
		aPV.addValue("true");
		aPV.addValue("false");
		genericSettings.put("considerProteinNames", aPV);				
		//considerGeneNames
		aPV=new PermittedValues();
		aPV.addValue("true");
		aPV.addValue("false");
		genericSettings.put("considerGeneNames", aPV);				
		//writePeptInfo
		aPV=new PermittedValues();
		aPV.addValue("true");
		aPV.addValue("false");
		genericSettings.put("writePeptInfo", aPV);				
		//encoding_output
		aPV=new PermittedValues();
		aPV.addValue("UTF-8");
		aPV.addValue("Latin-1");
		genericSettings.put("encoding_output", aPV);
		perValComplete.put("genericSettings", genericSettings);
	}
	
	/**
	 * This method generates value references for the section 'commandLineSettings' of the configuration file.
	 */
	private void generateComLineReferences(){
		PermittedValues aPV;
		//compType
		aPV=new PermittedValues();
		aPV.addValue("acc");
		aPV.addValue("sequence");
		aPV.addValue("peptide");
		commandLineSettings.put("compType", aPV);
		perValComplete.put("commandLineSettings", this.commandLineSettings);		
	}
	
	/**
	 * This method generates value references for the section 'grouping' of the configuration file.
	 */
	private void generateGroupReferences(){
		PermittedValues aPV;
		//selection_representative
		aPV=new PermittedValues();
		aPV.addValue("longestSequence");
		aPV.addValue("random");
		grouping.put("selection_representative", aPV);
		perValComplete.put("grouping", grouping);			
	}
	
	/**
	 * This method generates value references for the section 'Seq. based comparison' of the configuration file.
	 */
	private void generateSeqBasedReferences(){
		PermittedValues aPV;
		//PID_method
		aPV=new PermittedValues();
		aPV.addValue("PID1");
		aPV.addValue("PID2");
		aPV.addValue("PID3");
		aPV.addValue("PID4");
		sequenceBasedComparison.put("PID_method", aPV);
		//substitution_matrix_id
		aPV=new PermittedValues();
		aPV.addValue("BLOSUM30");
		aPV.addValue("BLOSUM35");
		aPV.addValue("BLOSUM40");
		aPV.addValue("BLOSUM45");
		aPV.addValue("BLOSUM50");
		aPV.addValue("BLOSUM55");
		aPV.addValue("BLOSUM60");
		aPV.addValue("BLOSUM62");
		aPV.addValue("BLOSUM65");
		aPV.addValue("BLOSUM70");
		aPV.addValue("BLOSUM75");
		aPV.addValue("BLOSUM80");
		aPV.addValue("BLOSUM85");
		aPV.addValue("BLOSUM90");
		aPV.addValue("BLOSUM100");
		aPV.addValue("PFAM250");
		sequenceBasedComparison.put("substitution_matrix_id", aPV);
		//alignment_method
		aPV=new PermittedValues();
		aPV.addValue("NW");
		sequenceBasedComparison.put("alignment_method", aPV);
		perValComplete.put("Seq. based comparison", sequenceBasedComparison);				
	}
	
	/**
	 * This method generates value references for the section 'Peptide based comparison' of the configuration file.
	 */
	private void generatePeptBasedReferences(){
		PermittedValues aPV;
		//peptideBasedMethod
		aPV=new PermittedValues();
		aPV.addValue("includeSequenceInfo");
		aPV.addValue("peptidesOnly");
		peptideBasedComparison.put("peptideBasedMethod", aPV);
		//consideration_of_modifications
		aPV=new PermittedValues();
		aPV.addValue("true");
		aPV.addValue("false");
		peptideBasedComparison.put("consideration_of_modifications", aPV);
		perValComplete.put("Peptide based comparison", peptideBasedComparison);			
	}
	
	//Getter
	/**
	 * The method returns a Map with key-value pairs where wrong settings have been 
	 * applied in the ProLiC configuration file.
	 * @return a Map with key-value pairs 
	 */
	public HashMap<String, String> getWrongValues(){
		return this.wrongValues;
	}
	
	/**
	 * The method returns a Map with key-value pairs where 
	 * a) wrong settings have been applied in the ProLiC configuration file and
	 * b) default values are applied in order to prevent ProLiC from shutdown.
	 * @return a Map with key-value pairs 
	 */
	public HashMap<String, String> getAppliedDefaultValues(){
		return this.AppliedDefaultValues;
	}
}
