/**
 * 
 */
package de.mpc.prolic.misc;
/**
* @author Michael Kohl
* The source code of this class is modified but in general subject to:
* Copyright (c) 2002 Stefan Matthias Aust. All Rights Reserved.
*
* This class facilitate reading properties from files given in the .ini file style and provides
* an interface to read string, integer and boolean values.  
* .ini files have the following structure:
* <pre>; a comment
* [section]
* key=value</pre>
* entrys without to lowerCase
* 
* Original Copyright:
* 
*  * Copyright (c) 2002 Stefan Matthias Aust.  All Rights Reserved.
*
* You are granted the right to use this code in a) GPL based projects in 
* which case this code shall be also protected by the GPL, or b) in other 
* projects as long as you make all modifications or extensions to this 
* code freely available, or c) make any other special agreement with the 
* copyright holder.
*/

import java.io.*;
import java.util.*;

public class IniReader {

//Member
	private Map <String,Map<String,String>> sections = new HashMap <String,Map<String,String>>();

//Constructors
	public IniReader(String pathname) throws FileNotFoundException, IOException {
		this(new FileReader(pathname));
	}

	public IniReader(InputStream input) throws FileNotFoundException, IOException {
		this(new InputStreamReader(input));
	}

	public IniReader(Reader input) throws FileNotFoundException, IOException {
		BufferedReader read=new BufferedReader(input);
		initialize(read);
		read.close();
	}

/**
* Reading the content from an .ini file and transform this content in a Map structure.
* @param r
* @throws IOException
*/
	private void initialize(BufferedReader r) throws IOException {
		String section = null, line;
		while ((line = r.readLine()) != null) {
			line = line.trim();
			if (line.equals("") || line.startsWith(";")) {
				continue;
			}
			if (line.startsWith("[")) {
				if (!line.endsWith("]")) {
				    throw new IOException("] expected in section header");
				}
			    //section = line.substring(1, line.length() - 1).toLowerCase();
				section = line.substring(1, line.length() - 1);
			} else if (section == null) {
			    throw new IOException("[section] header expected");
			} else {
				int index = line.indexOf('=');
				if (index < 0) {
				    throw new IOException("key/value pair without =");
				}
			    //String key = line.substring(0, index).trim().toLowerCase(); //original
			    String key = line.substring(0, index).trim();
			    String value = line.substring(index + 1).trim();
			    if(value.equals("\\t")){ //Modifikation MK: Because Java reads two characters: a backslash and a t. Therefore, replacement is necessary
			    	value="\t";
			    }
			    Map<String,String> map = sections.get(section);
				if (map == null) {
				    sections.put(section, (map = new HashMap<String,String>()));
				}
				map.put(key, value);
			}
		}
	}

//Setter and Getter
/**
* This method returns a specific string value stored in an ini-File with respect to a given section and a key
* @param section
* @param key
* @param defaultValue
* @return a String value stored in the ini-style ProLiC configuration file selected with respect to a given section - key combination.   
*/
	public String getPropertyString(String section, String key, String defaultValue) {
		Map <String,String> map = sections.get(section.toLowerCase());
		if (map != null) {
			//String value = (String) map.get(key.toLowerCase());//original
			String value = (String) map.get(key);
			if (value != null) {
				return value;
			}
		}
		return defaultValue;
	}

/**
* This method returns a specific integer value stored in an ini-File with respect to a given section and a key
* @param section
* @param key
* @param defaultValue
* @return an int value stored in the ini-style ProLiC configuration file selected with respect to a given section - key combination.   
*/
	public int getPropertyInt(String section, String key, int defaultValue) {
		String s = getPropertyString(section, key, null);
		if (s != null) {
			return Integer.parseInt(s);
		}
		return defaultValue;
	}

/**
* This method returns a specific boolean value stored in an ini-File with respect to a given section and a key
* @param section
* @param key
* @param defaultValue
* @return a boolean value stored in the ini-style ProLiC configuration file selected with respect to a given section - key combination.   
*/
	public boolean getPropertyBool(String section, String key, boolean defaultValue) {
		String s = getPropertyString(section, key, null);
		if (s != null) {
			if (s.equalsIgnoreCase("true")) {
				return true;
			}
			return false;
		}
		return defaultValue;
	}

/**
* This method returns the Map structure that gives for each key (= section name of the ini file)
* a Map that consists of further key-value pairs (configuration parameters)
* @return a Map that includes further Maps with all key-value pairs of the ProLiC configuration file.
*/
	public Map <String,Map<String,String>> getMap() {
		return this.sections;
	}

}
