/**
 * 
 */
package de.mpc.prolic.misc;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;
import java.util.ResourceBundle;
//import java.util.ResourceBundle;
import java.util.TreeSet;

//import org.apache.logging.log4j.LogManager;
//import org.apache.logging.log4j.Logger;

import java.util.regex.MatchResult;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import de.mpc.prolic.exceptions.WrongProLiCSettingException;
import de.mpc.prolic.model.Biomolecule;
import de.mpc.prolic.model.BiomoleculeGroup;
import de.mpc.prolic.model.BiomoleculeList;
import de.mpc.prolic.model.CleavageSite;
import de.mpc.prolic.model.Enzyme;
import de.mpc.prolic.model.Peptide;
import de.mpc.prolic.model.Protein;


/**
 * @author Michael Kohl
 * Class that contains static methods for some common procedures used in ProLiC.
 */
public class Tools {
	

	private final static ResourceBundle bundleMess = ResourceBundle.getBundle("de.mpc.prolic.resources.MessageBundle");	//A localized bundle for user information and for logging
	//Intended for future usage:
	//private static final Logger log4j = LogManager.getLogger(Tools.class.getName());
	//private static final boolean debugMode = true;
	
    /*******************************************************************************
    * Static methods for string manipulations
    ******************************************************************************/
	
	/**
	 * Trims the file extension and returns the remaining name of the file. In case there wasn't any '.', the string given as transfer parameter is returned.
	 * @param filename
	 * @return a filename where the file extension was removed. 
	 */
    public static String trimFileExtension (String filename) {
        int pos = filename.lastIndexOf(".");
        // In case there wasn't any '.' just return the string as it is.
        if (pos == -1) return filename;
        // Otherwise return the string, up to the last dot.
        return filename.substring(0, pos);
    }
    
    /**
     * Gets the extension from the specified name of the file. In case there isn't any '.' null is returned.
     * @param filename
     * @return the extension obtained from a given filename.
     */
    public static String getFileExtension (String filename) {
    	String extension=null;
    	int i = filename.lastIndexOf('.');   	
   		if (i > 0) {    	
   			extension = filename.substring(i+1);
	    }
    	return extension;
    }
    
    /**
     * Removes the modification information from a given sequence with respect to the applied notation of modifications.
     * The resulting string solely consists of a sequence of amino acids in the one letter code. In case that a notation of 
     * modifications is specified that is not supported by ProLiC an exception is thrown and the unmodified sequence is maintained. 
     * The method is currently not in use.
     * @param sequence
     * @param modificationNotation
     * @return a protein sequence string where the post translational modification information was removed.
     */ 
    public static String removeModifications(String sequence, String modificationNotation) throws WrongProLiCSettingException{
    	boolean wrongModificationNotation=false;
    	String adjustedString="";
    	String lowerCaseRegEx="[a-z]"; 				//Detects all lower case characters.
    	String roundBracketsRegEx="\\(.*?\\)";		//Detects all characters that are surrounded with round brackets.
    	String squareBracketsRegEx="\\[.*?\\]";		//Detects all characters that are surrounded with square brackets.
    	String numbersRegEx="\\d*\\.\\d*";			//Detects all numbers given within 'sequence'.
    	Matcher matcher=null;
    	StringBuffer sb;
    	if (modificationNotation.equals("lowercase_character")){
    		matcher = Pattern.compile(lowerCaseRegEx).matcher( sequence );    		
    	}else{
        	if (modificationNotation.equals("round_brackets")){
        		matcher = Pattern.compile(roundBracketsRegEx).matcher( sequence );  
        	}else{
            	if (modificationNotation.equals("square_brackets")){
            		matcher = Pattern.compile(squareBracketsRegEx).matcher( sequence );  
            	}else{
                	if (modificationNotation.equals("numbers")){
                		matcher = Pattern.compile(numbersRegEx).matcher( sequence );
                	}else{
                		wrongModificationNotation=true;
                		throw new WrongProLiCSettingException();
                	}
            	}
        	}
    	}
    	if(!wrongModificationNotation){
        	sb= new StringBuffer(sequence.length());
    		while (matcher.find())
    				  matcher.appendReplacement( sb, "" );//Replace all detected characters by nothing
    		matcher.appendTail( sb );
    		adjustedString=sb.toString();
        	return adjustedString;    		
    	}else{
    		
    		return sequence; //Returns the unmodified sequence
    	}
    }
    
    /**
     * Removes the modification information from a given sequence. The resulting string solely consists of a sequence of amino acids in the one letter code.
     * @param sequence
     * @param regExPTM
     * @return a protein sequence string (amino acid one letter code) where the post translational modification information was removed.
     */
    public static String removePTMs(String sequence, String regExPTM) {
    	String adjustedString="";
    	Matcher matcher=null;
    	StringBuffer sb;
    	matcher = Pattern.compile(regExPTM).matcher( sequence );
       	sb= new StringBuffer(sequence.length());
   		while (matcher.find())
   				  matcher.appendReplacement( sb, "" );//Replace all detected characters by nothing
   		matcher.appendTail( sb );
   		adjustedString=sb.toString();
       	return adjustedString;    		
    }

    /**
     * Removes the modification information from all peptides in a given peptide set.
     */
    public static ArrayList<Peptide> removeModificationsFromPeptideSet(ArrayList<Peptide> aPeptSet, String regExPTM){
		ArrayList<Peptide> aPeptSetNoMod=new ArrayList<Peptide>();
		Peptide aPept;
		Peptide aPeptNoMod;
		for(int k=0; k<aPeptSet.size();k++){
			aPept=aPeptSet.get(k);
			aPeptNoMod=new Peptide();
			aPeptNoMod.setSequence(Tools.removePTMs(aPept.getSequence(), regExPTM));
			aPeptSetNoMod.add(aPeptNoMod);
		}
		return aPeptSetNoMod;
    }
    
    /*******************************************************************************
    * Static methods for sequence based comparisons
    ******************************************************************************/
    
    /**
     * Calculates the number of terminal gaps for the given string that represents the result of a pairwise sequence alignment.
     * The  given program parameter is a string that contains a line separator. Therefore, the given string can be split into substrings
     * that represent the alignment result for the query and the target sequence.   
     * @param alignmentRes
     * @return the number of external gaps calculated from the results of a pairwise sequence alignment.
     */
    public static int calculateExternalGaps(String alignmentRes){
    	Pattern pn = Pattern.compile("^-*"); //regular expression for N-terminal gaps
    	Pattern pc = Pattern.compile("-*$"); //regular expression for C-terminal gaps
    	int numbNTermGaps=0;
    	int numbCTermGaps=0;
    	String[] split;
    	String queryRes, targetRes;
    	//Splitting alignmentRes
    	split=alignmentRes.split("\n");
    	queryRes=split[0];
    	targetRes=split[1];
    	//Terminal gaps for the query sequence
    	Matcher mn = pn.matcher(queryRes);
    	if (mn.find()){
    		numbNTermGaps=mn.end()-mn.start();
    	} 
    	Matcher mc = pc.matcher(queryRes);
    	if (mc.find()){
    		numbCTermGaps=mc.end()-mc.start();
    	} 
    	//Terminal gaps for the target sequence
    	mn = pn.matcher(targetRes);
    	if (mn.find()){
    		numbNTermGaps=numbNTermGaps+mn.end()-mn.start();
    	} 
    	mc = pc.matcher(targetRes);
    	if (mc.find()){
    		numbCTermGaps=numbCTermGaps+mc.end()-mc.start();
    	}  
    	return (numbNTermGaps+numbCTermGaps);
    }
    
    /**
     * The method calculates the number of gaps for a given sequence obtained for an alignment algorithm
     * @param seq
     * @return the number of gaps calculated from the results of a pairwise sequence alignment.
     */
    public int calculateNumberGaps(String seq){
    	int res=0;
    	
    	return res;    	
    }
    
    
    /*******************************************************************************
    * Static methods for peptide based comparisons
    *******************************************************************************/
	/**
	 * Calculates the maximum number of peptides that are assigned to a protein within the given BiomoleculeList. This method is only applicable for 
	 * protein lists.
	 * @return the maximum number of peptides for the proteins of a biomolecule list. 
	 */
	public static int calcMaxNumberPeptids(BiomoleculeList aBL){
		int maxNumbPeptides=0;
		int currMaxNumbPeptides=0;
		Protein aBM;
		for (int i=0;i<aBL.getBiomolecules().size();i++){
			aBM=(Protein)aBL.getBiomolecules().get(i);
			currMaxNumbPeptides=aBM.getPeptides().size();
			if (currMaxNumbPeptides>maxNumbPeptides){
				maxNumbPeptides=currMaxNumbPeptides;
			}
		}
		return maxNumbPeptides;
	}
	
		
	/**
	 * This method creates an instance of Enzyme from the information specified in the Enzymes section of the configuration file.
	 * @param enzymeFeatures
	 * @return an instance of Enzyme.
	 */
	public static Enzyme createEnzyme(String enzymeFeatures) throws WrongProLiCSettingException{
		String messWrongDirection=bundleMess.getString("m_access_error_settings_enzymes");
		Enzyme e = new Enzyme();
		CleavageSite aCS;
		String[] enzymePropertiesArr = enzymeFeatures.split("_");
		String enzymeName=enzymePropertiesArr[0];								//The name of the used enzyme.
		char[] aa_sites=enzymePropertiesArr[1].toCharArray(); 					//char[] of the cleavage sites (amino acids in one letter code)					
		String cleavageDirection=enzymePropertiesArr[2]; 						//this String specifies the direction where the enzyme cleaves (N-terminal or C-terminal)
		e.setRegExDigestion(enzymePropertiesArr[enzymePropertiesArr.length-1]); //The regular expression used to determine in silicio digested peptides (in particular: this regular expression is used to calculate the terminal peptide from a protein sequence)
		e.setEnzymeName(enzymeName);
		for(int i=0;i<aa_sites.length;i++){
			aCS=new CleavageSite();
			aCS.setAa_site(aa_sites[i]);
			e.addCleavageSite(aCS);
		}
		if (cleavageDirection.equals("N")){
			e.setPerformsN_terminalCleavage(true);
		}else{
			if (cleavageDirection.equals("C")){
				e.setPerformsN_terminalCleavage(false);
			}else{
				throw new WrongProLiCSettingException(messWrongDirection);
			}
		}
		return e;		
	}
	
	/**
	 * This method returns a list of matched subsequences according to the regular expression pattern specified as parameter of the method.
	 * The method is currently not in use.
	 * @param pattern
	 * @param s
	 * @return a list of matched subsequences calculated from a a given sequence using a regular expression.
	 */
	public static List<MatchResult> findMatches( String pattern, CharSequence s ){
		  List<MatchResult> results = new ArrayList<MatchResult>();
		  for ( Matcher m = Pattern.compile(pattern).matcher(s); m.find(); )
		    results.add( m.toMatchResult() );
		  return results;
	}
	
    /*******************************************************************************
    * Static methods used for grouping
    *******************************************************************************/
	
	/**
	 * Building a list identifier for this BiomoleculeGroup
	 * @param aBG
	 * @return a list identifier for a given biomolecule group.
	 */
	public static String buildListIdentifier(BiomoleculeGroup aBG){
		char intersection_sign='\u2229'; //intersection sign
		String assembledListIdentifier="";
		int counter=1;
		String aSingleListIdentifier;
		Biomolecule aBm;
		TreeSet<ListIdentifier> ts = new TreeSet<ListIdentifier>();
		Iterator <Biomolecule> it=aBG.getBiomoleculeGroup().iterator();
		Iterator <ListIdentifier> it2;
		while(it.hasNext()){
			aBm=it.next();
			ts.add(new ListIdentifier(aBm.getOrigin()));
		}
		it2=ts.iterator();
		while (it2.hasNext()){
			aSingleListIdentifier=it2.next().getListIdentifier();
			assembledListIdentifier=assembledListIdentifier+aSingleListIdentifier; 
			if (counter<ts.size()){
				assembledListIdentifier=assembledListIdentifier+" "+intersection_sign+" ";
			}
			counter++;
		}
		return assembledListIdentifier;
	}  	
	
    /*******************************************************************************
    * Miscellaneous
    *******************************************************************************/
	
    /**
     * Gives a formatted string representation of the current date and time. Enables a global alteration of the formatting for date and time outputs. 
     * @return a formatted string representation of the current date and time.
     */
    public static String getCurrentDate(){
    	String dateAsString;
    	Calendar c = new GregorianCalendar();
    	dateAsString = c.getTime().toString();
    	return dateAsString;
    }
}
